/* ***** BEGIN LICENSE BLOCK *****
 *
 * SmoothWheel: Smart algorithms for smooth scrolling with the mouse wheel.
 * Copyright (C) 2003 Avi Halachmi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * contact:  avihpit (*at*) yahoo (*dot*) com
 *
 * The Original Code is SmoothWheel.
 *
 * The Initial Developer of the Original Code is Avi Halachmi.
 * Portions created by the Initial Developer are Copyright (C) 2003 Avi Halachmi.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


/*********************************************************
 *
 * SmoothWheel by Avi Halachmi (avihpit (*at*) yahoo (*dot*) com)
 * http://smoothwheel.mozdev.org
 * Complete changelog, Info and PR statement is at the bottom of this file.
 *
 * started  (Avi Halachmi): 22-Mar-2003 (v0.1)
 *
 * modified (Avi Halachmi): 15-Apr-2003 (v0.2):
 *  - Improved Algorithm, Improved Timing, much improved target detection,
 *    Modifier keys, Better configuration and presets, Optimizations.
 *
 * modified (Avi Halachmi): 22-May-2003 (v0.3):
 *  - new target detection code (courtesy of All-In-One-Gestures author), "disable" Modifier key,
 *    soft-edge, adaptive-step, adaptive-duration, works smoothly on view-src and composer as well,
 *    namespace improvements, more event handler optimizations for the default settings, some more presets.
 *
 *********************************************************/


//**************** CONFIGURATION SECTION ****************

//Global enable/disable for this extension. default is true.
// setting it to false will disable this extension completely.
const sw_EnableThisExtension=true;

// select one of several preset settings: (default is 7)
// 0 : don't use preset. use actual variables values.
// 1 : turbo:  for those who like quickies  (duration=150, non-linear, smooth transitions, relative step)
// 2 : fast: slower than turbo, faster than the medium   (duration=300, non-linear, smooth transitions, relative step)
// 3 : medium: [was the default in v0.2] (duration=500, non-linear, smooth transitions, relative step)
// 4 : nice and easy: yet even slower and smoother   (duration=1000, non-linear, smooth transitions, relative step)
// 5 : fast + adaptive-step
// 6 : fast + adaptive-duration
// 7 :  -> [DEFAULT] fast + adaptive-step + adaptive-duration
// 8 : medium + adaptive-step
// 9 : same as version 0.1 default (duration=250, linear, no smooth transition, no relative step)
// 10 : same as version 0.1 with isLinear=false, duration=150 (no smooth transition, no relative step)
const sw_PresetSettings=7;

//max milisec to complete scroll after last scroll event
//reasonable values are (fast)100/250/500/1000(slow). default is 300
// -> this value is overriden if a preset is selected.
var   sw_ScrollDuration=300;

//if enabled, set step size relative to the scrolled area height by a constant factor.
//thus, if enabled and the visibla window/frame is short, so will be the scroll steps.
//when enabled, it overrides gecko internal fixed pixels/line constant. default is true.
// -> this value is overriden if a preset is selected.
var   sw_EnableRelativeStepSize=true;

//if enabled, the step size will vary according to the delay between consecutive wheel events.
//such that fast wheel scroll will cause step size to increase, and vice versa.
//the desired effect is that when you're reading, and want to scroll, the step is small,
//but it you're just browsing through the document, with faster wheel action,
//you'll gain more 'milage' per wheel event. default is true.
// -> this value is overriden if a preset is selected.
var   sw_EnableAdaptiveStep=true;

//if enabled, duration will vary according to the delay between consecutive wheel events,
//such that fast wheel scroll will cause duration to DEcrease, and vice versa.
//the desired effect is to make it slower and smoother while reading, but 'snappy' when browsing.
//default is true.
// -> this value is overriden if a preset is selected.
var   sw_EnableAdaptiveDuration=true;

//if enabled, causes the scroll to 'land gracefully' on the top or bottom edges of the document.
//default is true.
var   sw_EnableSoftEdge=true;

//Limits the frames/sec of the smooth scroll operation.
//for best visual performance, set this value to your monitor refresh rate. default is 100.
//reasonable values are 20 to 120
//note that setting it to high values will make no difference if your cpu isn't up for the task.
//tips: for faster Gecko rendering, set Desktop color to 16bpp, and turn off font-anti-aliasing
//IMPORTANT TIP: if possible, set your monitor refresh rate to 100, and ScrollMaxFPS to either 100 or 50.
//   (other values will cause imprecision due to low resolution internal timers)
const sw_ScrollMaxFPS=100;

//IMPORTANT NOTE ABOUT MODIFIER KEYS:
//by default, mozilla/phoenix/firebird assigns the ctrl key to enlarge/reduce the text size.
//the alt key is assigned to browse history. shift key is not assigned.
//in order for a modifier key to work with this extension, it has to be UNassigned first.
//you can UNassign a modifier key for the mouse wheel as follows:
//mozilla: edit->preferences->advanced->mouse Wheel -> set it to scroll normally.
//phoenix/firebird: about:config in the locationbar bar. then change mousewheel.with<xxx>key.action property to 0

//Select a modifier key for fine small scroll steps. default is 3 (SHIFT key).
//possible values: 0=Disable, 1=ALT, 2=CTRL, 3=SHIFT
const sw_ModifierSmallStep=3;

//Select a modifier key for full page scroll (like PgUp/PgDn). default is 1 (ALT key).
//possible values: 0=Disable, 1=ALT, 2=CTRL, 3=SHIFT
const sw_ModifierFullPageStep=1;

//Select a modifier key for currently disabling smoothwheel. default is 0 (not assigned).
//possible values: 0=Disable, 1=ALT, 2=CTRL, 3=SHIFT
const sw_ModifierDisableSmoothWheel=0;

//curtesy of All-In-One-Gestures author, starting with v0.3, smoothwheel uses a new target detection system
//(that's the system that decides what part of the screen to scroll when the wheel is scrolled)
//the new system is able to scroll inside textareas of forms, drop-down menus and more.
//however, on older Gecko builds, it currently performs less that optimal. So, if you feel that you prefere
//previous version's (v0.2) detection mechanism, set sw_compatibilityDetectionMode to true.
//default is false.
const sw_compatibilityDetectionMode=false;

//********* END OF COMMON CONFIGURATION OPTIONS SECTION ***********

//WHILE THE ABOVE COMMON CONFIGURATIONS ARE MOSTLY ENABLE/DISABLE, THE NEXT
//VALUES CAN BE USED TO FINE-TUNE THE BEHAVIOUR OF EACH FEATURE

//these 2 values control the step size when EnableAdaptiveStep=true.
//they define the factors for the smallest and largest steps, respectively. defaults are .4 and 1.2
//for extreme control (and variation) (requires a steady hand as well), try 0.3 and 3 respectively.
var sw_AdaptiveStepMinStepFactor=.4;
var sw_AdaptiveStepMaxStepFactor=1.2;

//these 2 values control the duration when EnableAdaptiveDuration=true.
//they define the factors for the smallest and largest durations, respectively. defaults are .8 and 2
var sw_AdaptiveDurationMinStepFactor=.8;
var sw_AdaptiveDurationMaxStepFactor=2;

//factor to use when EnableRelativeStepSize is true. default is 0.15 of viewable page height.
const sw_RelativeStepFactor=0.15;

//this factor configure how much "full page" is when ModifierFullPageStep is used.
//1 will result in full page height. 0.5 results in half page, etc.
//default is 0.9 of full page height.
const sw_FullPageFactor=0.9;

//factor for how short the scroll steps are if ModifierSmallStep is used.
//default is 0.1 of normal step.
const sw_ShortStepFactor=0.1;

//#pixels per scroll line. relevant only if EnableRelativeStepSize=false.
//default value is 20 (empirical estimation of internal Gecko value)
const sw_ScrollPx=20;

//the next 4 values control the duration between wheel events that define the
//adaptive behaviour of the adaptiveStep and adaptiveDurations features.

//defaults are .7 and 1.5
var sw_AdaptiveStepMinDurationFactor=.7;
var sw_AdaptiveStepMaxDurationFactor=1.5;

//defaults are .7 and 1.5
var sw_AdaptiveDurationMinDurationFactor=.7;
var sw_AdaptiveDurationMaxDurationFactor=1.5;

//******* it's recommended not to change the next values since it'll prevent optimizations *******/

//tries to make a smooth transition if new scroll event occures while still scrolling. default is true.
// -> this value is overriden if a preset is selected.
// -> changing the default value will prevent code optimizations.
var   sw_EnableSmoothEventTransition=true;

//selects either linear (=0) or various non-linear(=1/2/3) scroll profiles. default is 1.
//possible values:
// 0 : Linear
// 1 : 2nd power (non-linear)
// 2 : 3rd power (non-linear)
// 3 : cosine    (non-linear)
// -> this value is overriden if a preset is selected.
// -> changing the default value will prevent code optimizations.
var   sw_SmoothScrollProfile=1;

//percentage of when to finish transitional profile (from previous velocity to absolute scroll profile)
//default is 1.
// -> this value is overriden if a preset is selected.
// -> changing the default value will prevent code optimizations.
var   sw_TransEnd=1;

//THIS VALUE IS IRRELEVANT WITH THE NEW (v0.3+) DETECTION MECHANISM
//Whether or not to enable WHOLE PAGE scroll when mouse wheel is over form textarea. default is false.
// if set to false, mouse wheel events will NOT be intercepted by this extension while
//   mouse is over a form textarea. instead, the DEFAULT scroll will be issued.
// if set to true, the WHOLE PAGE will scroll smoothly when mouse is over a form textarea.
const sw_EnableWholePageScrollInForms=false;

//*************** END OF CONFIGURATION SECTION ********************


const sw_LoopInterval=Math.round(1000/sw_ScrollMaxFPS); //fps converted to ms between scroll events

//Global variables (used by more than one function).
var sw_scrollLeft=0; //in pixels
var sw_scrollLeftLastEvent=0; //in pixels
var sw_scrollEventTimeStamp=0;
var sw_scrollDeadline;// timestamp
var sw_clientFrame=null;  //holds the object to scroll
var sw_intervalID=null;   //holds the interval timer handle. also indicates whether we're currently scrolling.
var sw_velocityLastEvent=0;
var sw_velocityCurrent=0;
var sw_scrollCurrentStep;  //how many pixels to scroll for the current scroll event.
var sw_scrollCurrentDuration;  //how many ms to assign for the current scroll event.
var sw_optimizedProfileCodeExists; //whether optimized code exist according to various variables
var sw_optimizedHandlerExists=false;//if all vairables match, there's a pre-set optimized handler for default settings.
var sw_maxTop;// maximum top to scroll to. =sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight
var sw_isScrollable=false;


//checks whether the specified modifier key was held when the event was intercepted.
function sw_modifierHeld(e, modifier){
    switch (modifier) {
       case 1: return e.altKey;
       case 2: return e.ctrlKey;
       case 3: return e.shiftKey;
    }
    return false;
}


function sw_findTargetObject(e){
  if (sw_compatibilityDetectionMode)
       return e.target.ownerDocument.defaultView;
  //implicit else. next code is adapted from All-In-One-Gestures.

  var scrollsVert=false,
      targetDoc = e.target.ownerDocument,
      clientFrame = targetDoc.defaultView,
      insertionNode = (targetDoc.documentElement) ? targetDoc.documentElement : targetDoc,
      docBox = targetDoc.getBoxObjectFor(insertionNode),
      nextNode = e.target,
      bodyEl = targetDoc.getElementsByTagName("body")[0],
      currNode;

  if (bodyEl) { // walk the tree up looking for something to scroll
     if (nextNode.nodeName.toLowerCase() == "html") nextNode = bodyEl;
     do {
        try {
          currNode = nextNode;
          nextNode = currNode.parentNode;
          if (currNode.clientWidth && currNode.clientHeight) {
             if  (currNode.scrollHeight > currNode.clientHeight){
                scrollsVert=true;
                break;
             }
          }
        }
        catch(err) {}
     }  while (currNode && currNode != bodyEl);
  }
  sw_isScrollable=scrollsVert;
  return currNode;
}

function sw_findTargetViewHeight(target){
  if (sw_compatibilityDetectionMode)
     return target.innerHeight;

  //implicit else
  return target.clientHeight;
}

function sw_scrollVert(target, y){
  if (sw_compatibilityDetectionMode)
     return target.scrollBy(0,y);

  //implicit else
  target.scrollTop += y;

}

function sw_isValidTarget(target){
  if (sw_compatibilityDetectionMode)
     return true;//target.scrollBy(0,y);

  return sw_isScrollable;
}


//called when wheel event captured, while NOT scrolling.
//determines whether to leave it alone, or to use it (and initilize the vars)
function sw_acquireEvent(e){
    if (sw_modifierHeld(e, sw_ModifierDisableSmoothWheel)) return;
    var parents= sw_parentTree(e.target);
    var tmpClientFrame= sw_findTargetObject(e);//e.target.ownerDocument.defaultView;

    //1st, check if we should intercept this event at all, or let the default handler do it's stuff.
    if (((!sw_compatibilityDetectionMode)&&(!sw_isValidTarget(e))) ||
        (sw_compatibilityDetectionMode)&&
         (
          tmpClientFrame != "[object Window]" ||
          parents.indexOf("HTMLTextAreaElement")>=0  && (!sw_EnableWholePageScrollInForms) ||
          parents.indexOf("XULElement")>=0 ||
          parents.indexOf("HTMLEmbedElement")>=0 ||
          //sw_parents.indexOf("HTMLImageElement")>=0 ||
          parents.indexOf("HTMLOptionElement")>=0
         )
        )
        return false; //default handler will take it from here.

    //implicit else: ok. intercepting event. initialize variables, step size etc.
    sw_intervalID= window.setInterval("sw_smoothWheelLoop()",sw_LoopInterval);
    sw_clientFrame= tmpClientFrame;

    var viewHeight= sw_findTargetViewHeight(sw_clientFrame);//clientFrame.innerHeight;
    if (sw_modifierHeld(e, sw_ModifierFullPageStep))
       sw_scrollCurrentStep=viewHeight*sw_FullPageFactor;
    else
       sw_scrollCurrentStep=(sw_EnableRelativeStepSize)? viewHeight*sw_RelativeStepFactor: sw_ScrollPx*(Math.abs(e.detail));

    if (sw_modifierHeld(e, sw_ModifierSmallStep))
       sw_scrollCurrentStep= sw_scrollCurrentStep*sw_ShortStepFactor;

    sw_maxTop=sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight;

    sw_velocityCurrent=0;
    return true;
}


// set some of the adaptive functionality parameters only once in advance.
var sw_ASa=((sw_AdaptiveStepMinStepFactor-sw_AdaptiveStepMaxStepFactor)/
        (sw_AdaptiveStepMaxDurationFactor-sw_AdaptiveStepMinDurationFactor))/sw_ScrollDuration;//diagonal
var sw_ASb=sw_AdaptiveStepMaxStepFactor-sw_ASa*sw_AdaptiveStepMinDurationFactor; //offset

var sw_ADa=((sw_AdaptiveDurationMaxStepFactor-sw_AdaptiveDurationMinStepFactor)/
        (sw_AdaptiveDurationMaxDurationFactor-sw_AdaptiveDurationMinDurationFactor))/sw_ScrollDuration;//diagonal
var sw_ADb=sw_AdaptiveDurationMinStepFactor-sw_ADa*sw_AdaptiveDurationMinDurationFactor; //offset


//SmoothWheel wheel event handler, registered for handling DOMMouseScroll events
function smoothWheelHandler(e){
      if (sw_optimizedHandlerExists){// if all settings match, use a pre-configured handler.
        if (!sw_intervalID) if (!sw_acquireEvent(e))  return;

        var now= (new Date()).getTime(), dtn=(now-sw_scrollEventTimeStamp), sf=sw_ASa*dtn+sw_ASb, df=sw_ADa*dtn+sw_ADb;
        sw_scrollCurrentDuration=300*((df<.8)?.8:((df>2)?2:df));
        sw_scrollLeft += sw_scrollCurrentStep*((e.detail>0)?1:-1)*((sf>1.2)?1.2:((sf<.4)?.4:sf));
        var st=sw_clientFrame.scrollTop, sd=st+sw_scrollLeft;
        sw_scrollLeft=(sd<0)?-st:((sd>sw_maxTop)?sw_maxTop-st:sw_scrollLeft);
        sw_scrollEventTimeStamp=now;
        sw_scrollDeadline= sw_scrollEventTimeStamp + sw_scrollCurrentDuration;
        sw_scrollLeftLastEvent= sw_scrollLeft;
        sw_velocityLastEvent= sw_velocityCurrent;
        e.preventDefault();
        e.stopPropagation();
        return;
      }
      //implicit else, consider all variables explicitly

      if (!sw_intervalID) //if not currently scrolling
         if (!sw_EnableThisExtension || !sw_acquireEvent(e)) //if event should not be intercepted
            return;  //let default handler do it's stuff
         //implicit else: event acquired and vars initialized

      e.preventDefault();
      e.stopPropagation();

      var now= (new Date()).getTime();

      var dtn=(now-sw_scrollEventTimeStamp);

      var suggestedStepFactor=sw_ASa*dtn+sw_ASb;
      if (suggestedStepFactor>sw_AdaptiveStepMaxStepFactor) suggestedStepFactor=sw_AdaptiveStepMaxStepFactor;
      else if (suggestedStepFactor<sw_AdaptiveStepMinStepFactor) suggestedStepFactor=sw_AdaptiveStepMinStepFactor;


      var suggestedDurationFactor=sw_ADa*dtn+sw_ADb;
      if (suggestedDurationFactor<sw_AdaptiveDurationMinStepFactor)  suggestedDurationFactor=sw_AdaptiveDurationMinStepFactor;
      else if (suggestedDurationFactor>sw_AdaptiveDurationMaxStepFactor)  suggestedDurationFactor=sw_AdaptiveDurationMaxStepFactor;


      sw_scrollCurrentDuration=sw_ScrollDuration*((sw_EnableAdaptiveDuration)?suggestedDurationFactor:1);

      sw_scrollLeft += ((e.detail>0)?1:-1)*sw_scrollCurrentStep*((sw_EnableAdaptiveStep)?suggestedStepFactor:1);

      if (sw_EnableSoftEdge){
         var destination=sw_clientFrame.scrollTop+sw_scrollLeft;
         if (destination < 0) sw_scrollLeft = -sw_clientFrame.scrollTop;
         else if (destination > sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight)
            sw_scrollLeft = sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight - sw_clientFrame.scrollTop
      }

      sw_scrollEventTimeStamp=now;
      sw_scrollDeadline= sw_scrollEventTimeStamp + sw_scrollCurrentDuration;
      sw_scrollLeftLastEvent= sw_scrollLeft;
      sw_velocityLastEvent= sw_velocityCurrent;
}


//loop for smooth scrolling the page
function sw_smoothWheelLoop(){
      var toScroll, currentTime, currentPercentage, timeLeft;
      currentTime= (new Date()).getTime();
      timeLeft= sw_scrollDeadline - currentTime;

      if (timeLeft<=0)  //we're past the deadline. scroll what's left.
          toScroll=sw_scrollLeft;
      else{ //calculate next scroll destination according to scroll profile
          currentPercentage= 1 - timeLeft/sw_scrollCurrentDuration;
          toScroll= Math.round (sw_scrollLeftLastEvent*(sw_scrollProfile(currentPercentage) - 1) + sw_scrollLeft);
          if ((sw_scrollLeft-toScroll)*sw_scrollLeft < 0) //if going past destination point
              toScroll=sw_scrollLeft;
      }

      sw_scrollVert(sw_clientFrame, toScroll);
      sw_scrollLeft -= toScroll;
      if (sw_scrollLeft==0){//no more scroll needed
         window.clearInterval(sw_intervalID);
	 sw_intervalID=null;
         sw_velocityCurrent=0;
         return;
      }
}

//vars for optimized profile only.
var sw_percentage, sw_p1, sw_p2, sw_p21, sw_p2v;


//transform percentage of time to percentage of location according to a scroll profile
//it might consider an implicit parameter of the speed of the scroll at the time the event was intercepted.
function sw_scrollProfile(percentage){
        if (sw_optimizedProfileCodeExists){
             sw_p=percentage; sw_p1=1-sw_p; sw_p2=sw_p1*sw_p1; sw_p21=1-sw_p2; sw_p2v=sw_p2*sw_velocityLastEvent;
             sw_velocityCurrent= sw_p2v + sw_p21*2*sw_p1;
             return sw_p2v*sw_p + sw_p21*sw_p21; //location
        }
        // the above optimized code (hardcoded values, inlined functioned) is used if:
        //    SmoothScrollProfile=1 (->profilePower2) and EnableSmoothEventTransition=true and TransEnd=1
        // the optimized code does the stuff instead of running fully (sometimes multiple times) through the next functions:
        //    scrollProfile(p), transition(p,x,y),
        //    profilePower2(p), profilePower2Velocity(p),
        //    profileConstVelocity(p), profileConstVelocityVelocity(p)
        // the best optimization though would be to get 100-200 precalculated results into a table, and merge the value with velocityLastEvent in runtime;)
        //optimized transition(p,x,y)=p2*x+p21*y //p2 and p21 are already functions of p

        //implicit else: non-optimized path, consider all settings explicitly.
        var location, velocity;
        switch (sw_SmoothScrollProfile){
               case 0:  location= sw_profileLinear(percentage); break;
               case 1:  location= sw_profilePower2(percentage); break;
               case 2:  location= sw_profilePower3(percentage); break;
               case 3:  location= sw_profileCosine(percentage); break;
               default: location= sw_profileLinear(percentage); break;
        }
        if (sw_EnableSmoothEventTransition){
           switch (sw_SmoothScrollProfile){
               case 0:  velocity= sw_profileLinearVelocity(percentage); break;
               case 1:  velocity= sw_profilePower2Velocity(percentage); break;
               case 2:  velocity= sw_profilePower3Velocity(percentage); break;
               case 3:  velocity= sw_profileCosineVelocity(percentage); break;
               default: velocity= sw_profileLinearVelocity(percentage); break;
           }
           location= sw_transition(percentage, sw_profileConstVelocity(percentage), location);
           sw_velocityCurrent= sw_transition(percentage, sw_profileConstVelocityVelocity(percentage), velocity);
        }
        return location;
}


//returns a weighted average of 2 profiles according (non-linear) to percentage.
function sw_transition(percentage, srcProfile, destProfile){
         var srcPer= ((sw_TransEnd-percentage)/sw_TransEnd)*((sw_TransEnd-percentage)/sw_TransEnd);
         var destPer= 1-srcPer;
         return (percentage < sw_TransEnd)?
                        srcPer*srcProfile + destPer*destProfile: destProfile;
}


//various scroll profiles and their velocities (derivatives) functions.
function sw_profileLinear(percentage){return percentage;}
function sw_profileLinearVelocity(percentage){return 1;}

function sw_profilePower2(percentage){return 1-(1-percentage)*(1-percentage);}
function sw_profilePower2Velocity(percentage){return 2*(1-percentage);}

function sw_profilePower3(percentage){return 1-(1-percentage)*(1-percentage)*(1-percentage);}
function sw_profilePower3Velocity(percentage){return 3*(1-percentage)*(1-percentage);}

function sw_profileCosine(percentage){return .5-.5*Math.cos(Math.PI*percentage);}
function sw_profileCosineVelocity(percentage){return .5*Math.sin(Math.PI*percentage)*Math.PI;}

//using implicit parameter velocityLastEvent.
//this profile does NOT achieve destination point on deadline.
//it represents a linear motion at the speed that was recorded when the event was intercepted.
//this profile is used for transition when a scroll event is intercepted while still scrolling.
function sw_profileConstVelocity(percentage){return sw_velocityLastEvent*percentage;}
function sw_profileConstVelocityVelocity(percentage){return sw_velocityLastEvent;}


// start: functions used by old target detection code
function sw__generalParentTree(node, parentStringElement, counter){
  if (!node)
     return "("+counter+") [object null("+parentStringElement+")]";

  return ""+sw__generalParentTree(eval("node."+parentStringElement), parentStringElement, counter+1)+ "," + node;
}

function sw_generalParentTree(baseNode, selectionStr, parentElementStr){
         return selectionStr+": "+sw__generalParentTree(eval("baseNode."+selectionStr), parentElementStr, 0);
}

//return a string of all parentNode elements (recursively) for the input node
function sw_parentTree(node){
  if (!node)
     return "[object null(generatedString)]";

  return ""+sw_parentTree(node.parentNode)+ "," + node;
}
// end: functions used by old target detection code


//initialize variable according to the preset, and determine whether optimization is possible.
function sw_initSmoothWheel(){
    //[ScrollDuration, EnableRelativeStepSize, EnableAdaptiveStep, EnableAdaptiveDuration, SmoothScrollProfile, EnableSmoothEventTransition, TransEnd]
    var presets=[, [150,  true,  false, false,  1, true,  1], //1
                   [300,  true,  false, false,  1, true,  1], //2
                   [500,  true,  false, false,  1, true,  1], //3 : was default in v0.2
                   [1000, true,  false, false,  1, true,  1], //4
                   [300,  true,  true,  false,  1, true,  1], //5
                   [300,  true,  false, true ,  1, true,  1], //6
                   [300,  true,  true , true ,  1, true,  1], //7 : default with v0.3
                   [500,  true,  true , false , 1, true,  1], //8
                   [250,  false, false, false,  0, false, 1], //9
                   [150,  false, false, false,  2, false, 1]  //10
                 ];

    if (sw_PresetSettings >= 1 && sw_PresetSettings <=10){
       sw_ScrollDuration=              presets[sw_PresetSettings][0];
       sw_EnableRelativeStepSize=      presets[sw_PresetSettings][1];
       sw_EnableAdaptiveStep=          presets[sw_PresetSettings][2];
       sw_EnableAdaptiveDuration=      presets[sw_PresetSettings][3];
       sw_SmoothScrollProfile=         presets[sw_PresetSettings][4];
       sw_EnableSmoothEventTransition= presets[sw_PresetSettings][5];
       sw_TransEnd=                    presets[sw_PresetSettings][6];
    }

    sw_optimizedProfileCodeExists= (sw_SmoothScrollProfile==1) && (sw_EnableSmoothEventTransition==true) &&  (sw_TransEnd==1);

    sw_optimizedHandlerExists = (
        sw_EnableThisExtension &&
        sw_EnableAdaptiveStep &&
        sw_EnableAdaptiveDuration &&
        sw_EnableSoftEdge &&
        (sw_ScrollDuration==300) &&
        (sw_AdaptiveStepMinStepFactor==0.4) &&
        (sw_AdaptiveStepMaxStepFactor==1.2) &&
        (sw_AdaptiveDurationMinStepFactor==.8) &&
        (sw_AdaptiveDurationMaxStepFactor==2)
    );
}

//runs once, when the file is loaded.
sw_initSmoothWheel();


// ********** start: debug/test functions *************

// display a debug text on the javascript console.
function dbg(t){
//         that's the best way i found so far. i assume there's a better one though.
        window.setTimeout("DBG: -->" +t+ "<--",10);
}

// ********** end: debug/test functions *************



/***************************************************************************
SmoothWheel extension for Mozilla/Phoenix/Firebird/Netscape
SmoothWheel home page: http://smoothwheel.mozdev.org
Download latest version: http://downloads.mozdev.org/smoothwheel/smoothwheel.xpi
Licence: GPL 2.0
Copyright 2003 Avi Halachmi (avihpit (*at*) yahoo (*dot*) com)
Current version: v0.3 (22-May-2003)

About:
The smoothwheel extension will scroll the page smoothly with the mouse wheel.
SmoothWheel utilizes several smart algorithms for smoothness and adaptive behaviour.
It can enhance/replace the internal Gecko smoothscroll which was introduced on 2003-03-24.
Tested platforms: win32, Linux, FreeBsd/i386, Mac OS/X (theoretically should work on all platforms).
It's also working with older versions of Mozilla/Netscape/Phoenix/Firebird.

Features:
- Can be disabled easily, even in Mozilla or Netscape.
- Automatically utilizes strong CPUs for smoother scroll.
- Completely configurable, including 10 presets.
- Scroll steps relative to scrollable area height (configurable).
- Advanced Optimized algorithm for a VERY smooth scroll in many scenarios.
- Soft-Edge : Lands gracefully on top/bottom of document.
- Adaptive-Step : Shorter duration between wheel events -> more pixels/event.
- Adaptive-Duration : Shorter duration between wheel events -> scrolls faster.
- Modifier keys for small/full page steps (SHIFT and ALT respectively by default).
- Modifier key for temporarily disabling smoothwheel (none by default).
- Timer based operation: Always 'finishes on time'.
- Lets the default mouse wheel handler work if smoothwheel can't handle it.

Todo:
- Make it work on XUL panes (mail-and-news top/left panes, irc left/bottom panes, etc).
- Possibly improve installation.

/////////////////////////////////////////////////////////////////////////////
changelog:
/////////////////////////////////////////////////////////////////////////////
v0.3 (22-May-2003)

bugFix (v0.3):
- new target detection code, courtesy of All-In-One-Gestures author.
- cleanup of namespace (precaution measures)

new features (v0.3):
- now working on view source (all browsers) and composer.
- modifier key for temporarily disabling SmoothWheel.
- Soft-Edge : Lands gracefully on top/bottom of document.
- Adaptive-Step : Shorter duration between wheel events -> more pixels/event.
- Adaptive-Duration : Shorter duration between wheel events -> scrolls faster.
- more optimizations for event handler if default settings are kept.
- some more presets to reflect the new features.

status (v0.3):

scrolling SMOOTHLY on:
- mozilla: browser, forms, view-source, mail-and-news (message windows), composer, irc (main window)
- phoenix/firebird: browser, forms, view-source

letting default scroll handler do it's stuff on:
- mozilla: top/left panes of mail-and-news, left/bottom irc panes, all XUL panes, about:config
- phoenix/firebird: about:config, all XUL panes

breaking wheel scroll functionality on (v0.2):
- couldn't find any so far.

/////////////////////////////////////////////////////////////////////////////
v0.2 (15-Apr-2003)

bugFix (v0.2):
- much better target detection code. will let default scroll handler do it's stuff if we can't handle properly.
- better timing of scroll loop
- fps limit (for fast CPUs)

new features (v0.2):
- modifier keys for slow/fullPage scroll
- improved algorithm: smooth transition if scroll event occures while still scrolling
- much more configuration options (including 6 presets)
- optimized code for the most common settings


status (v0.2):

scrolling SMOOTHLY on:
- mozilla: browser, view-source, irc (main window), mail-and-news (message windows)
- phoenix: browser

letting default scroll handler do it's stuff on:
- mozilla: browser forms, some mail-and-news panes, some irc panes,
           composer, all XUL panes, about:config, more..
- phoenix: browser forms, view-source, about:config, all XUL panes, more..

breaking wheel scroll functionality on (v0.2):
- couldn't find any so far.

/////////////////////////////////////////////////////////////////////////////
v0.1 (22-Mar-2003)

status (v0.1):

scrolling SMOOTHLY on:
- mozilla: browser, view-source, irc (main window), mail-and-news (message windows)
- phoenix: browser

letting default scroll handler do it's stuff on:
- mozilla: composer
- phoenix: view-source

breaking wheel scroll functionality on:
- mozilla: browser forms, some mail-and-news panes, some irc panes, all XUL panes, more...
- phoenix: browser forms, all XUL panes, more...
***************************************************************************
*/
